{
  inputs = {
    haskell-nix.url = "github:input-output-hk/haskell.nix/11337f2fb85cd606630929f8fce485160d343cdc";
    nixpkgs.follows = "haskell-nix/nixpkgs-unstable";
    flake-utils.url = "github:numtide/flake-utils";
    accelerate = { url = "git+https://github.com/AccelerateHS/accelerate.git?submodules=1"; flake = false; };
    accelerate-arithmetic = { url = "github:alpmestan/accelerate-arithmetic"; flake = false; };
    accelerate-utility = { url = "git+https://gitlab.iscpif.fr/amestanogullari/accelerate-utility.git?ref=fix-error"; flake = false; };
    eigen = { url = "github:chessai/eigen/8fff32a43df743c8c83428a86dd566a0936a4fba"; flake = false; };
    haskell-igraph = { url = "git+https://gitlab.iscpif.fr/gargantext/haskell-igraph.git"; flake = false; };
    hmatrix = { url = "github:alpmestan/hmatrix/alp/imap-static"; flake = false; };
    sparse-linear = { url = "github:alpmestan/sparse-linear/alp/static"; flake = false; };
  };
  outputs = { self, ... }@inputs:
    let
      supportedSystems = with inputs.flake-utils.lib.system; [
        x86_64-linux
      ];
    in
    inputs.flake-utils.lib.eachSystem supportedSystems (system:
      let
        pkgs = import inputs.nixpkgs {
          inherit system;
          config = inputs.haskell-nix.config;
          overlays = [
            inputs.haskell-nix.overlay
            (finalPkgs: previousPkgs: {
              # Warning: the nixpkgs pinned brings igraph-0.10.7
              # not the igraph-0.10.4 expected by haskell-igraph-10.0.4
              igraph = (previousPkgs.igraph.override {
                # fop pulls openjdk, scipy, blas… just to build PDF docs,
                fop = null;
              }).overrideAttrs (previousAttrs: {
                buildInputs = with finalPkgs; [
                  #arpack
                  glpk
                  gmp
                  libxml2
                  openblas
                  plfit
                ];
                cmakeFlags = /*previousAttrs.cmakeFlags ++*/ [
                  # Enable shared build.
                  "-DBUILD_SHARED_LIBS=ON"
                  # Make nm work without a plugin for LTO.
                  "-DIGRAPH_ENABLE_LTO=OFF"
                  "-DIGRAPH_USE_INTERNAL_BLAS=OFF"
                  "-DIGRAPH_USE_INTERNAL_LAPACK=OFF"
                  # Using an external ARPACK does not work
                  # later on when linking haskell-igraph-test-test
                  # due to undefined references (eg. to dsortr_).
                  # Even when arpack is built with -DBUILD_SHARED_LIBS=OFF,
                  # is some other dependency not statically built? gfortran?
                  "-DIGRAPH_USE_INTERNAL_ARPACK=ON"
                  "-DIGRAPH_USE_INTERNAL_GLPK=OFF"
                  "-DIGRAPH_USE_INTERNAL_GMP=OFF"
                  "-DIGRAPH_USE_INTERNAL_PLFIT=OFF"
                  "-DIGRAPH_GLPK_SUPPORT=ON"
                  "-DIGRAPH_GRAPHML_SUPPORT=ON"
                  "-DIGRAPH_OPENMP_SUPPORT=OFF"
                  "-DIGRAPH_ENABLE_TLS=ON"
                  #"-DIGRAPH_VERIFY_FINALLY_STACK=ON"
                ];
                doCheck = false;
              });
              openblas = previousPkgs.openblas.override {
                # See https://github.com/OpenMathLib/OpenBLAS/issues/2543
                singleThreaded = true;
              };
              plfit = previousPkgs.plfit.overrideAttrs (previousAttrs: {
                cmakeFlags = previousAttrs.cmakeFlags ++ [
                  "-DPLFIT_USE_OPENMP=OFF"
                ];
              });
              pkgsCross = previousPkgs.pkgsCross // {
                musl64 = previousPkgs.pkgsCross.musl64.extend (finalMusl64: previousMusl64: {
                  # Warning: the nixpkgs pinned brings igraph-0.10.7
                  # not the igraph-0.10.4 expected by haskell-igraph-10.0.4
                  igraph = previousMusl64.igraph.overrideAttrs (previousAttrs: {
                    buildInputs = (previousAttrs.buildInputs or []) ++ (with finalMusl64; [
                      gomp
                    ]);
                    cmakeFlags = previousAttrs.cmakeFlags ++ [
                      # Enable static build.
                      "-DBUILD_SHARED_LIBS=OFF"
                      # Using a static openblas cause multiple segfaults in igraph tests.
                      "-DIGRAPH_USE_INTERNAL_BLAS=ON"
                      "-DIGRAPH_USE_INTERNAL_LAPACK=ON"
                      # Enabling link time optimization fails with a lot of undefined references.
                      "-DIGRAPH_ENABLE_LTO=OFF"
                      #"-DIGRAPH_VERIFY_FINALLY_STACK=ON"
                      #"-DCMAKE_CXX_FLAGS=-fPIC"
                      #"-DCMAKE_C_FLAGS=-fPIC"
                    ];
                    doCheck = true;
                  });
                  openblas = previousMusl64.openblas.override {
                    enableStatic = true;
                    enableShared = true;
                    # See https://github.com/OpenMathLib/OpenBLAS/issues/2543
                    singleThreaded = true;
                  };
                  lapack-reference = previousMusl64.lapack-reference.override {
                    shared = false;
                  };
                  blas-reference = previousMusl64.blas-reference.overrideAttrs (previousAttrs: {
                    cmakeFlags = previousAttrs.cmakeFlags ++ [
                      # Enable static build.
                      "-DBUILD_SHARED_LIBS=OFF"
                    ];
                    doCheck = false;
                  });
                  arpack = previousMusl64.arpack.overrideAttrs (previousAttrs: {
                    cmakeFlags = previousAttrs.cmakeFlags ++ [
                      # Enable static build.
                      "-DBUILD_SHARED_LIBS=OFF"
                      #"-DCMAKE_CXX_FLAGS=-fPIC"
                      #"-DCMAKE_C_FLAGS=-fPIC"
                    ];
                  });
                });
              };
            })
          ];
        };
        inherit (inputs.nixpkgs) lib;
        inherit (pkgs.haskell-nix) haskellLib;

        project = pkgs.haskell-nix.cabalProject' {
          name = "gargantext-graph";
          src = ./.;

          compiler-nix-name = "ghc948";
          #compilerSelection = p: p.haskell-nix.compiler;
          index-state ="2023-12-10T10:34:46Z";
          inputMap = {
            "https://github.com/AccelerateHS/accelerate.git" = inputs.accelerate;
            "https://github.com/alpmestan/accelerate-arithmetic.git" = inputs.accelerate-arithmetic;
            "https://github.com/alpmestan/hmatrix.git" = inputs.hmatrix;
            "https://github.com/alpmestan/sparse-linear.git" = inputs.sparse-linear;
            "https://github.com/chessai/eigen.git" = inputs.eigen;
            "https://gitlab.iscpif.fr/amestanogullari/accelerate-utility.git" = inputs.accelerate-utility;
            "https://gitlab.iscpif.fr/gargantext/haskell-igraph.git" = inputs.haskell-igraph;
          };

          shell = {
            exactDeps = true;
            #packages = ps: lib.attrValues (haskellLib.selectProjectPackages ps);
            withHoogle = false;
            nativeBuildInputs = [
              #pkgs.pkg-config
              (project.tool "cabal" { version = "latest"; })
            ];
          };

          modules = [
            ( { pkgs, ...}: {
              packages.haskell-igraph.components.library = {
                # The generated plan includes pkgs.igraph
                # but pkgs.igraph.dev is also needed for igraph.h
                libs = [ pkgs.igraph.dev ];
                # Extra include needed because haskell-igraph's cbits
                # use: #include <igraph.h>
                # not: #include <igraph/igraph.h>
                configureFlags = [ "--extra-include-dirs=${pkgs.igraph.dev}/include/igraph" ];
              };
            })
          ];

          flake.variants = {
            static = {
              modules = [
                ({ pkgs, pkgconfPkgs, config, options, ... }: {
                  enableStatic = true;

                  # src/Language/Haskell/TH/Instances/Internal.hs:12:1: error:
                  #     Failed to load dynamic interface file for Control.Monad.Trans:
                  #     Bad interface file: /nix/store/d1z23zmjjrcrjbah1rl34haw1bbgr37w-mtl-lib-mtl-x86_64-unknown-linux-musl-2.2.2/lib/x86_64-linux-ghc-9.4.8/mtl-2.2.2-JH2R61IXO86L7oCsewaz7T/Control/Monad/Trans.dyn_hi
                  #         /nix/store/d1z23zmjjrcrjbah1rl34haw1bbgr37w-mtl-lib-mtl-x86_64-unknown-linux-musl-2.2.2/lib/x86_64-linux-ghc-9.4.8/mtl-2.2.2-JH2R61IXO86L7oCsewaz7T/Control/Monad/Trans.dyn_hi: openBinaryFile: does not exist (No such file or directory)
                  #    |
                  # 12 | import qualified Control.Monad.Trans as MTL (lift)
                  #    | ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
                  #
                  # AFAICS, it does not apply DYNAMIC_GHC_PROGRAMS=NO to GHC.
                  #enableShared = false;

                  # error: attribute 'defaultSetupFor' missing
                  #ghc.package = options.ghc.package.default.override { enableShared = false; };

                  # See https://github.com/haskell-numerics/hmatrix/issues/307
                  packages.hmatrix.flags.no-random_r = true;
                  # See https://github.com/input-output-hk/haskell.nix/issues/914#issuecomment-2021507590
                  packages.hmatrix.flags.openblas = true;

                  # Due to unknown symbol __cpu_model in bitvec-1.1.5.0
                  # when using -fexternal-interpreter,
                  # the SIMD optimization has to be disabled.
                  # See https://github.com/input-output-hk/haskell.nix/issues/2047
                  packages.bitvec.flags.simd = false;

                  # Fix c2hs failing to run ghc
                  # because when cross-compiling it's prefixed, eg. x86_64-unknown-linux-musl-ghc
                  # TODO: remove when https://github.com/haskell/c2hs/pull/293 has been merged
                  packages.c2hs.components.exes.c2hs.depends = [
                    # If not there:
                    # Error: Setup: Encountered missing or private dependencies
                    config.hsPkgs.ghc-paths
                  ];
                  packages.c2hs.patches = [
                    patches/c2hs/0001-fix-GHC-path-to-support-cross-compiling.patch
                  ];
                  # Alas, line-ending mess on c2hs.cabal prevents to use a patch.
                  # And patchFlags=["--binary"] is not passed to mkDerivation by haskell.nix
                  # Hence use substituteInPlace to add ghc-paths to the build-depends.
                  packages.c2hs.postPatch = ''
                    substituteInPlace c2hs.cabal \
                      --replace "base >= 2 && < 5," "base >= 2 && < 5, ghc-paths,"
                  '';

                  packages.haskell-igraph.ghcOptions = [
                    #"-optl=-L${pkgs.arpack}/lib"
                  ];
                  packages.haskell-igraph.configureFlags = [
                    #"--extra-lib-dirs=${pkgs.arpack}/lib"
                  ];

                  # gargantext-graph uses TemplateHaskell.
                  #
                  # Not using -fexternal-interpreter fails with:
                  #
                  # <no location info>: error:
                  #      Error relocating /nix/store/g1crf4blkprrry58sjx29glvcvd811cp-haskell-igraph-lib-haskell-igraph-x86_64-unknown-linux-musl-0.10.4/lib/x86_64-linux-ghc-9.4.8/libHShaskell-igraph-0.10.4-EBe5Nu77ckCJAIVDVPknAF-ghc9.4.8.so: dsortr_: symbol not found
                  packages.gargantext-graph.ghcOptions = [
                    #"-fexternal-interpreter"
                  ];
                  packages.gargantext-graph.configureFlags = [
                    "-v"
                    "-v"
                  ];
                })
              ];
            };
          };

          flake.crossPlatforms = platforms:
            pkgs.lib.optionals pkgs.stdenv.hostPlatform.isx86_64 (
              pkgs.lib.optionals pkgs.stdenv.hostPlatform.isLinux [
                platforms.musl64
              ]
            );
        };
      in lib.recursiveUpdate project.flake' {
        legacyPackages = pkgs // { inherit project; };
      }
    );

  nixConfig = {
    extra-substituters = [ "https://cache.iog.io" ];
    extra-trusted-public-keys = [ "hydra.iohk.io:f/Ea+s+dFdN+3Y/G+FDgSq+a5NEWhJGzdjvKNGv0/EQ=" ];
    allow-import-from-derivation = "true";
  };
}
